	#  A simple program demonstrating self-modifying code.  When this
	# program is executed, the add instruction just before label L1
	# below dynamically modifies the offset for the backward jump of
	# the loop, so that the target of the jump changes with each iteration,
	# until eventually execution breaks out of the loop.
	#
	# If self-modification occurs the way it's supposed to, the program
	# prints out the no. of iterations executed by the loop between
	# labels L0 and L1:
	#
	#       no. of iterations = 2
	
	.text
	.section	.rodata
.LC0:
	.string	"no. of iterations = %d\n"
	.text
	.globl	main
	.type	main, @function
main:
.LFB0:
	.cfi_startproc
	endbr64
	pushq	%rbp
	.cfi_def_cfa_offset 16
	.cfi_offset 6, -16
	movq	%rsp, %rbp
	.cfi_def_cfa_register 6
	xor %rax, %rax
L0:
	leaq L2(%rip), %rcx
	leaq L0(%rip), %rdx
	sub %rdx, %rcx
	imul %rax, %rcx
	inc %rax

	# This leaq instruction puts the address of the jmp instruction at
	# label L1 into register rdx. 
	leaq L1(%rip), %rdx

	# Code self-modification takes place via the add instruction after this
	# comment.  This instruction modifies the jmp instruction labeled by L1
	# so that, through two iterations of the L0-L1 loop, this instruction is
	# changed to be no longer a jump instruction, at which point execution
	# falls out of the loop and into the call to printf.  If this add
	# instruction is commented out, the jmp instruction at label L1 remains
	# unmodified and the program goes into an infinite loop.
	add %cl, 1(%rdx)
L1:
	jmp L0
L2:
	movl	%eax, %esi
	leaq	.LC0(%rip), %rdi
	movl	$0, %eax
	call	printf@PLT
	
	nop
	movl	$0, %eax
	popq	%rbp

	.cfi_def_cfa 7, 8
	ret
	.cfi_endproc
.LFE0:
	.size	main, .-main
	.ident	"GCC: (Ubuntu 9.3.0-10ubuntu2) 9.3.0"
	.section	.note.GNU-stack,"",@progbits
	.section	.note.gnu.property,"a"
	.align 8
	.long	 1f - 0f
	.long	 4f - 1f
	.long	 5
0:
	.string	 "GNU"
1:
	.align 8
	.long	 0xc0000002
	.long	 3f - 2f
2:
	.long	 0x3
3:
	.align 8
4:
