// Code source: https://www.quora.com/C-programming-language-Can-
// you-write-a-C-program-to-demonstrate-a-self-modifying-code

#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>

int add_two(int input) {
  int sum = input + 2;    // <<< modified code location
  return sum;
}

// Given a pointer to x86-64 machine code, get_add_imm() loops
// through looking for an add-immediate-byte instruction (opcode 0x83).
// If it finds one, it returns the offset to the immediate operand
// of this instruction.  If it reaches a retq instruction (opcode 0xc3)
// it returns 0.
int get_add_imm(unsigned char *ptr) {
  unsigned char *init = ptr;
  while (*ptr != 0xc3) {    /* 0xc3 == retq: a return instruction */
    if (*ptr == 0x83) {  /* 0x83: add immediate byte */
      return (ptr - init) + 2;
    }
    ptr++;
  }

  return 0;
}

void patch_code() {
    int pageSize = sysconf(_SC_PAGE_SIZE);

    // unprotect the two pages of memory containing the code for
    // add_two();  set their permissions to read+write+execute
    mprotect((void*) ((((long) &add_two) / pageSize) * pageSize),
	     pageSize * 2,
	     PROT_READ | PROT_WRITE | PROT_EXEC);

    // get the address of the function add_two
    unsigned char* addInstrPtr = ((unsigned char*) &add_two);

    // find the offset to the byte to be modified
    int offset = get_add_imm(addInstrPtr);

    // Change the byte used in addition
    *(addInstrPtr+offset) = 3;  // <<< code modifying instruction
}

int main() {
  int out;

  // before code modification, add_two(2) returns the value 4.
  printf("before code modification: add_two() returns %d\n", add_two(2));
  
  patch_code();

  // after code modification, add_two(2) returns the value 5.
  printf("after code modification: add_two() returns %d\n", add_two(2));
  
  return 0;
}
